@@ -1,4 +1,4 @@
-# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -22,7 +22,7 @@ import os
 
 from reahl.component.config import Configuration, ConfigSetting
 from reahl.web.libraries import LibraryIndex, JQuery, JQueryUI, Underscore, HTML5Shiv, IE9, Reahl, Holder, Popper, \
-    Bootstrap4, ReahlBootstrap4Additions, JsCookie
+    Bootstrap4, ReahlBootstrap4Additions, JsCookie, PlotlyJS
 
 
 class WebConfig(Configuration):
@@ -67,13 +67,18 @@ class WebConfig(Configuration):
     session_lifetime = ConfigSetting(default=60*60*24*7*2,
                                      description='The time in seconds a user session will be kept after last use')
     idle_lifetime = ConfigSetting(default=60*60*2,
-                                  description='The time in seconds after which a user session will be considered idle - normal setting')
+                                  description='The time in seconds after which a user session will be considered not logged in anymore')
     idle_lifetime_max = ConfigSetting(default=60*60*24*7*2,
-                                      description='The time in seconds after which a user session will be considered idle - "forever" setting')
+                                      description='The time in seconds after which a user session will be considered not logged in anymore - when the user opted to stay logged in')
     idle_secure_lifetime = ConfigSetting(default=60*60,
                                          description='The time in seconds after which a secure session will be considered expired')
     debug_concurrency_hash = ConfigSetting(default=False,
                                    description='If True, replaces the concurrency hash with a long string indicating the values used in calculating the hash')
+    csrf_key = ConfigSetting(default='unsafekey',
+                             description='Used to sign CSRF tokens for a site. Set this to a secret long string unique to your deployment.',
+                             dangerous=True)
+    csrf_timeout_seconds = ConfigSetting(default=60*5,
+                                         description='Forms have to be submitted within this time after being rendered.')
                                        
     @property
     def secure_key_name(self):
@@ -84,5 +89,5 @@ class WebConfig(Configuration):
         # We create it here, so that each instance of a WebConfig will have its own LibraryIndex instance
         self.frontend_libraries = LibraryIndex(JQuery(), JsCookie(), JQueryUI(), Underscore(), HTML5Shiv(), IE9(), Reahl(), Holder(),
                                                Popper(),  # must be before Bootstrap in html script includes
-                                               Bootstrap4(), ReahlBootstrap4Additions())
+                                               Bootstrap4(), ReahlBootstrap4Additions(), PlotlyJS())
 
