@@ -16,6 +16,7 @@
 
 
 import json
+import re
 
 from webob import Request
 from sqlalchemy import Column, Integer
@@ -486,12 +487,13 @@ def test_rendering_of_form(web_fixture):
     fixture.context.request = Request.blank('/a/b?x=y', charset='utf8')
     actual = tester.render_html()
 
-    expected = '''<form id="test_channel" action="/a/b/_test_channel_method?x=y" data-formatter="/__test_channel_format_method" method="POST" class="reahl-form">''' \
+    expected = '''<form id="test_channel" action="/a/b/_test_channel_method\?x=y" data-formatter="/__test_channel_format_method" method="POST" class="reahl-form">''' \
                '''<div id="test_channel_hashes">'''\
+               '''<input name="test_channel-_reahl_csrf_token" id="id-test_channel-_reahl_csrf_token" form="test_channel" type="hidden" value=".*" class="reahl-primitiveinput">''' \
                '''<input name="test_channel-_reahl_database_concurrency_digest" id="id-test_channel-_reahl_database_concurrency_digest" form="test_channel" type="hidden" value="" class="reahl-primitiveinput">''' \
                '''</div>''' \
                '''</form>'''
-    assert actual == expected
+    assert re.match(expected, actual)
 
     # Case: without querystring
     fixture.context.request = Request.blank('/a/b', charset='utf8')
@@ -833,7 +835,9 @@ def test_event_names_are_canonicalised(web_fixture):
     browser = Browser(wsgi_app)
 
     # when the Action is executed, the correct arguments are passed
-    browser.post('/__myform_method', {'event.myform-an_event?some_argument=f~nnystuff': '', 'myform-_reahl_database_concurrency_digest':''})
+    browser.open('/')
+    csrf_token = browser.get_value('//input[@name="myform-_reahl_csrf_token"]')
+    browser.post('/__myform_method', {'event.myform-an_event?some_argument=f~nnystuff': '', 'myform-_reahl_database_concurrency_digest':'', 'myform-_reahl_csrf_token': csrf_token})
     assert model_object.received_argument == 'f~nnystuff'
 
 
@@ -874,7 +878,9 @@ def test_alternative_event_trigerring(web_fixture):
     browser = Browser(wsgi_app)
 
     # when POSTing with _noredirect, the Action is executed, but the browser is not redirected to /page2 as usual
-    browser.post('/__myform_method', {'event.myform-an_event?': '', '_noredirect': '', 'myform-_reahl_database_concurrency_digest':''})
+    browser.open('/')
+    csrf_token = browser.get_value('//input[@name="myform-_reahl_csrf_token"]')
+    browser.post('/__myform_method', {'event.myform-an_event?': '', '_noredirect': '', 'myform-_reahl_database_concurrency_digest':'', 'myform-_reahl_csrf_token': csrf_token})
     browser.follow_response()  # Needed to make the test break should a HTTPTemporaryRedirect response be sent
     assert model_object.handled_event
     assert browser.current_url.path != '/page2'
@@ -883,10 +889,8 @@ def test_alternative_event_trigerring(web_fixture):
     # the response is a json object reporting the success of the event and a new rendition of the form
     json_dict = json.loads(browser.raw_html)
     assert json_dict['success']
-
-    browser.open('/')
-    expected_html = browser.get_inner_html_for('//form[1]')
-    assert json_dict['widgets']['myform'].startswith(expected_html+'<script')
+    expected_html = '<div id="myform_hashes">'
+    assert json_dict['result']['myform'].startswith(expected_html)
 
 
 @with_fixtures(WebFixture)
@@ -910,11 +914,13 @@ def test_remote_field_validation(web_fixture):
 
     wsgi_app = fixture.new_wsgi_app(child_factory=MyForm.factory('myform'))
     browser = Browser(wsgi_app)
+    browser.open('/')
+    csrf_token_string = web_fixture.get_csrf_token_string(browser=browser)
 
-    browser.open('/_myform_validate_method?myform-a_field=invalid email address')
+    browser.open('/_myform_validate_method?myform-a_field=invalid email address', headers={'X-CSRF-TOKEN':csrf_token_string})
     assert browser.raw_html == '"a_field should be a valid email address"'
 
-    browser.open('/_myform_validate_method?myform-a_field=valid@email.org')
+    browser.open('/_myform_validate_method?myform-a_field=valid@email.org', headers={'X-CSRF-TOKEN':csrf_token_string})
     assert browser.raw_html == 'true'
 
 
@@ -939,11 +945,13 @@ def test_remote_field_formatting(web_fixture):
 
     wsgi_app = fixture.new_wsgi_app(child_factory=MyForm.factory('myform'))
     browser = Browser(wsgi_app)
+    browser.open('/')
+    csrf_token_string = web_fixture.get_csrf_token_string(browser=browser)
 
-    browser.open('/_myform_format_method?myform-a_field=13 November 2012')
+    browser.open('/_myform_format_method?myform-a_field=13 November 2012', headers={'X-CSRF-TOKEN':csrf_token_string})
     assert browser.raw_html == '13 Nov 2012'
 
-    browser.open('/_myform_format_method?myform-a_field=invaliddate')
+    browser.open('/_myform_format_method?myform-a_field=invaliddate', headers={'X-CSRF-TOKEN':csrf_token_string})
     assert browser.raw_html == ''
 
 
